// "Build Engine & Tools" Copyright (c) 1993-1997 Ken Silverman
// Ken Silverman's official web site: "http://www.advsys.net/ken"
// See the included license file "BUILDLIC.TXT" for license info.

#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define NEWMAPVERSION 7
#define MAXMENUFILES 1024

#define MAXSECTORS 1024
#define MAXWALLS 8192
#define MAXSPRITES 4096

#define MAXSTATUS 1024
#define MAXTILES 4096

#define MENUNAME_SIZE 16
static char menuname[MAXMENUFILES][MENUNAME_SIZE+1];
static long menunamecnt;

#pragma pack(push,1)

// Version 5
typedef struct
{
	unsigned short wallptr, wallnum;
	short ceilingpicnum, floorpicnum;
	short ceilingheinum, floorheinum;
	long ceilingz, floorz;
	signed char ceilingshade, floorshade;
	char ceilingxpanning, floorxpanning;
	char ceilingypanning, floorypanning;
	char ceilingstat, floorstat;
	char ceilingpal, floorpal;
	char visibility;
	short lotag, hitag;
	short extra;
} oldsectortype;

typedef struct
{
	long x, y;
	short point2;
	short picnum, overpicnum;
	signed char shade;
	short cstat;
	unsigned char xrepeat, yrepeat, xpanning, ypanning;
	short nextsector1, nextwall1;
	short nextsector2, nextwall2;
	short lotag, hitag;
	short extra;
} oldwalltype;

typedef struct
{
	long x, y, z;
	char cstat;
	signed char shade;
	unsigned char xrepeat, yrepeat;
	short picnum, ang, xvel, yvel, zvel, owner;
	short sectnum, statnum;
	short lotag, hitag;
	short extra;
} oldspritetype;

// Version 6
typedef struct
{
	unsigned short wallptr, wallnum;
	short ceilingpicnum, floorpicnum;
	short ceilingheinum, floorheinum;
	long ceilingz, floorz;
	signed char ceilingshade, floorshade;
	char ceilingxpanning, floorxpanning;
	char ceilingypanning, floorypanning;
	char ceilingstat, floorstat;
	char ceilingpal, floorpal;
	char visibility;
	short lotag, hitag, extra;
} sectortype;

typedef struct
{
	long x, y;
	short point2, nextsector, nextwall;
	short picnum, overpicnum;
	signed char shade;
	char pal;
	short cstat;
	unsigned char xrepeat, yrepeat, xpanning, ypanning;
	short lotag, hitag, extra;
} walltype;

typedef struct
{
	long x, y, z;
	short cstat;
	signed char shade;
	char pal, clipdist;
	unsigned char xrepeat, yrepeat;
	signed char xoffset, yoffset;
	short picnum, ang, xvel, yvel, zvel, owner;
	short sectnum, statnum;
	short lotag, hitag, extra;
} spritetype;

// Version 7

	//40 bytes
typedef struct
{
	short wallptr, wallnum;
	long ceilingz, floorz;
	short ceilingstat, floorstat;
	short ceilingpicnum, ceilingheinum;
	signed char ceilingshade;
	char ceilingpal, ceilingxpanning, ceilingypanning;
	short floorpicnum, floorheinum;
	signed char floorshade;
	char floorpal, floorxpanning, floorypanning;
	char visibility, filler;
	short lotag, hitag, extra;
} newsectortype;

	//32 bytes
typedef struct
{
	long x, y;
	short point2, nextwall, nextsector, cstat;
	short picnum, overpicnum;
	signed char shade;
	char pal, xrepeat, yrepeat, xpanning, ypanning;
	short lotag, hitag, extra;
} newwalltype;

	//44 bytes
typedef struct
{
	long x, y, z;
	short cstat, picnum;
	signed char shade;
	char pal, clipdist, filler;
	unsigned char xrepeat, yrepeat;
	signed char xoffset, yoffset;
	short sectnum, statnum;
	short ang, owner, xvel, yvel, zvel;
	short lotag, hitag, extra;
} newspritetype;

#pragma pack(pop)

//**************************************************************************

static oldsectortype oldsector[MAXSECTORS];
static oldwalltype oldwall[MAXWALLS];
static oldspritetype oldsprite[MAXSPRITES];

static sectortype sector[MAXSECTORS];
static walltype wall[MAXWALLS];
static spritetype sprite[MAXSPRITES];

static newsectortype newsector[MAXSECTORS];
static newwalltype newwall[MAXWALLS];
static newspritetype newsprite[MAXSPRITES];

static long posx, posy, posz, mapversion;
static short ang, cursectnum, numsectors, numwalls, numsprites;

main(int argc, char **argv)
{
	long i;

	if (argc != 2)
	{
		printf("CONVMAP%ld [filespec]                                            by Ken Silverman\n",NEWMAPVERSION);
		printf("This CONVMAP%ld.EXE converts map version %ld and %ld to map version %ld\n",NEWMAPVERSION,NEWMAPVERSION-2,NEWMAPVERSION-1,NEWMAPVERSION);
		exit(0);
	}
	menunamecnt = 0;
	getfilenames(argv[1]);
	sortfilenames();

	if (menunamecnt == 0)
	{
		printf("CONVMAP%ld [filespec]                                            by Ken Silverman\n",NEWMAPVERSION);
		printf("This CONVMAP%ld.EXE converts map version %ld and %ld to map version %ld\n",NEWMAPVERSION,NEWMAPVERSION-2,NEWMAPVERSION-1,NEWMAPVERSION);
		printf("File not found\n");
		exit(0);
	}

	printf("Converting map to map version %ld\n",NEWMAPVERSION);

	for(i=0;i<menunamecnt;i++)
		convmap(menuname[i]);
}

sectorofwall(short theline)
{
	short i, startwall, endwall, sucksect;

	sucksect = -1;
	for(i=0;i<numsectors;i++)
	{
		startwall = oldsector[i].wallptr;
		endwall = startwall + oldsector[i].wallnum - 1;
		if ((theline >= startwall) && (theline <= endwall))
		{
			sucksect = i;
			break;
		}
	}
	return(sucksect);
}

int convert_pics(int pic)
{
	static int conversions[4096];
	static int setup_conversions = 0;

	if (!setup_conversions)
	{
		int i;

		memset(conversions,0,sizeof(conversions));

		#define OLD_NEW(old,new) conversions[old] = new

		OLD_NEW(1,1195);	// A 'step'
		OLD_NEW(2,757);		// $ steel panal/door slightly rusted
		OLD_NEW(3,757);		// $ steel panal/door slightly rusted
		OLD_NEW(4,757);		// $ steel panal/door slightly rusted
		OLD_NEW(5,5);		// MusicAndSFX
		OLD_NEW(6,241);		// $ concrete wall
		OLD_NEW(7,241);		// $ concrete wall
		OLD_NEW(8,757);		// $ dark rusty steel panal
		OLD_NEW(9,241);		// $ concrete wall + grafiti
		OLD_NEW(10,755);	// $ white checker plate
		
		OLD_NEW(18,750);	// bricks

		OLD_NEW(21,757);	// metal wall

		OLD_NEW(32,3);		// Touchplate
		OLD_NEW(23,4);		// Activator Locked

		OLD_NEW(27,315);	// $

		OLD_NEW(31,2);		// Activator

		OLD_NEW(34,6);		// Locator

		OLD_NEW(49,336);	// water

		OLD_NEW(51,8);		// Master Switch

		OLD_NEW(57,1247);	// seenine

		OLD_NEW(60,51);		// coke can -> 'cola'
		OLD_NEW(61,51);		// coke can -> 'cola'
		OLD_NEW(62,51);		// coke can -> 'cola'
		OLD_NEW(63,51);		// coke can -> 'cola'
		OLD_NEW(64,51);		// coke can -> 'cola'
		OLD_NEW(65,51);		// coke can -> 'cola'
		OLD_NEW(66,51);		// coke can -> 'cola'
		OLD_NEW(67,51);		// coke can -> 'cola'

		OLD_NEW(73,401);

		OLD_NEW(75,0);		// * Front of petrol pump

		OLD_NEW(83,775);
		OLD_NEW(84,157);
		OLD_NEW(85,0);		// * Side of petrol pump
		OLD_NEW(86,914);	// maskwall3
		OLD_NEW(87,350);	// *
		OLD_NEW(88,350);	// *
		OLD_NEW(89,89);		// la
		OLD_NEW(90,3420);	// yelblack
		OLD_NEW(91,353);	// yelblack
		OLD_NEW(92,354);	// yelblack plat
		OLD_NEW(93,355);	// yelblack

		OLD_NEW(94,89);		// hills->la

		for (i = 97; i <= 122; i++)		// White Characters -> Silver
			conversions[i] = i + 2966 - 97;
		for (i = 123; i <= 148; i++)	// White Characters -> Silver
			conversions[i] = i + 2966 - 123;
		for (i = 149; i <= 158; i++)	// White Characters -> Silver
			conversions[i] = i + 2966 - 97;

		OLD_NEW(167,977);	// genericpole
		OLD_NEW(168,800);	// 
		OLD_NEW(169,801);	// 

		OLD_NEW(172,978);	// witcheshat
		OLD_NEW(173,979);	// hanglight

		OLD_NEW(175,1);		// Sector Effector
		OLD_NEW(176,805);
		OLD_NEW(176,805);
		OLD_NEW(178,504);	// solid glass
		OLD_NEW(179,503);	// glass

		OLD_NEW(191,981);
		OLD_NEW(192,981);

		OLD_NEW(195,983);	// street sign
		OLD_NEW(196,984);	// street sign
		OLD_NEW(197,985);	// street sign
		OLD_NEW(198,986);	// street sign
		OLD_NEW(199,987);	// street sign
		OLD_NEW(200,988);	// ??
		OLD_NEW(201,989);	// street sign

		OLD_NEW(203,813);	// road
		OLD_NEW(204,814);	// road
		OLD_NEW(205,815);	// road
		OLD_NEW(206,816);	// road
		OLD_NEW(207,817);	// road
		OLD_NEW(208,818);	// road
		OLD_NEW(209,182);

		OLD_NEW(215,56);	// air

		OLD_NEW(220,763);	// windows
		OLD_NEW(221,821);	// door
		OLD_NEW(222,822);	// door

		OLD_NEW(225,315);	// $

		OLD_NEW(227,186);
		OLD_NEW(228,187);
		OLD_NEW(229,188);
		OLD_NEW(230,189);
		OLD_NEW(231,191);
		OLD_NEW(232,191);

		OLD_NEW(265,621);	// camera
		OLD_NEW(266,622);	// camera
		OLD_NEW(267,623);	// camera
		OLD_NEW(268,624);	// camera
		OLD_NEW(269,625);	// camera

		OLD_NEW(310,10);	// GPSPEED

		OLD_NEW(331,504);	// * glass + words

		OLD_NEW(334,407);	// fansprite
		OLD_NEW(335,409);	// fansprite
		OLD_NEW(336,410);	// fansprite
		OLD_NEW(337,411);	// fansprite

		OLD_NEW(350,150);	// Door

		OLD_NEW(368,720);	// 

		OLD_NEW(373,836);	// 

		OLD_NEW(384,1000);	// hollywood bvld
		OLD_NEW(385,1001);	// vine bvld

		OLD_NEW(395,132);	// slideswitch up
		OLD_NEW(396,133);	// slideswitch down

		OLD_NEW(405,134);	// flickswitch up
		OLD_NEW(406,135);	// flickswitch down

		OLD_NEW(407,913);	// fence

		OLD_NEW(408,2120);	// femsoldier -> lizman
		OLD_NEW(409,2121);	// femsoldier -> lizman
		OLD_NEW(410,2122);	// femsoldier -> lizman

		OLD_NEW(490,2000);	// soldier ->pigcop
		OLD_NEW(491,2001);	// soldier ->pigcop
		OLD_NEW(492,2002);	// soldier ->pigcop

		OLD_NEW(570,560);	// mirror
		OLD_NEW(571,561);	// ???
		OLD_NEW(572,562);	// floor mirror

		OLD_NEW(573,504);	// glass
		OLD_NEW(574,505);	// glass
		OLD_NEW(575,506);	// glass
		OLD_NEW(576,507);	// glass
		OLD_NEW(577,508);	// glass
		OLD_NEW(578,509);	// glass

		OLD_NEW(579,1880);	// 'drone'
		OLD_NEW(580,1881);	// 'drone'
		OLD_NEW(581,1882);	// 'drone'
		OLD_NEW(582,1883);	// 'drone'
		OLD_NEW(583,1884);	// 'drone'
		OLD_NEW(584,1885);	// 'drone'
		OLD_NEW(585,1886);	// 'drone'

		OLD_NEW(604,365);	// $
		OLD_NEW(605,366);	// $

		OLD_NEW(627,1238);	// exploding barrel

		OLD_NEW(628,1034);	// horselight
		OLD_NEW(629,1035);	// horselight
		OLD_NEW(630,1036);	// horselight

		OLD_NEW(634,1015);	// grate

		OLD_NEW(677,584);	// sensor

		OLD_NEW(713,57);	// jetpack
		OLD_NEW(716,53);	// medikit
		OLD_NEW(717,55);	// steroids
		OLD_NEW(718,1349);	// holoduke
		OLD_NEW(719,56);	// air
		OLD_NEW(720,1349);	// holoduke
		OLD_NEW(721,26);	// pipebomb
		OLD_NEW(722,52);	// sixpack
		OLD_NEW(723,41);	// ammo???
		OLD_NEW(724,55);	// steroids
		OLD_NEW(726,56);	// air
		OLD_NEW(727,53);	// medikit
		OLD_NEW(728,57);	// jetpack

		OLD_NEW(740,152);	// Door
		OLD_NEW(741,153);	// Door

		for (i = 756; i <= 768; i++)	// Screens
			conversions[i] = i + 263 - 756;

		OLD_NEW(768,268);
		OLD_NEW(728,57);	// jetpack


		OLD_NEW(786,998);	// light
		OLD_NEW(787,999);	// light
		OLD_NEW(788,1007);	// neon bar
		OLD_NEW(789,1008);	// neon cocktail glass

		OLD_NEW(793,1009);	// neon arrows
		OLD_NEW(794,1010);	// neon arrows
		OLD_NEW(795,1011);	// neon arrows

		OLD_NEW(827,1022);	// hangooz

		OLD_NEW(839,1170);	// $ wooden panels

		OLD_NEW(842,741);	// windows
		OLD_NEW(843,743);	// windows
		OLD_NEW(844,370);	// wood
		OLD_NEW(845,370);	// wood
		OLD_NEW(846,370);	// wood

		OLD_NEW(900,355);	// $

		OLD_NEW(905,904);	// wooden horse
		OLD_NEW(906,905);	// wooden horse
		OLD_NEW(907,907);	// wooden horse
		OLD_NEW(908,908);	// tree
		OLD_NEW(909,909);	// tree

		OLD_NEW(960,570);	// ??

		OLD_NEW(982,189);	// windows

		OLD_NEW(990,725);	// windows

		OLD_NEW(993,725);	// windows
		OLD_NEW(994,725);	// windows

		OLD_NEW(1280,1550);	// shark

		for (i = 1320; i < 1329; i++)	// Wide Numbers
			conversions[i] = i + 640 - 1320;

		OLD_NEW(1536,1680);	// robot -> liztrooper
		
		OLD_NEW(1680,1680);	// robot -> liztrooper
		OLD_NEW(1681,1681);	// robot -> liztrooper
		OLD_NEW(1682,1682);	// robot -> liztrooper

		OLD_NEW(1820,1820);	// octabrain
		OLD_NEW(1821,1821);	// octabrain
		OLD_NEW(1822,1822);	// octabrain
	}

	if (pic == 32)
	{
		printf ("Touchplate\n");
	}

	return conversions[pic];
}

convmap(char *filename)
{
	long i, j;

	if (strchr(filename,'.') == 0) strcat(filename,".map");
	if ((i = loadoldboard(filename)) < 0)
	{
		if (mapversion == NEWMAPVERSION-2)
			if ((i = loadreallyoldboard(filename)) >= 0) goto ok;

		if (i == -1)
		{
			printf("%s not found.\n",filename);
		}
		if (i == -2)
		{
			if (mapversion == NEWMAPVERSION)
				printf("%s already map version %ld.\n",filename,NEWMAPVERSION);
			else
				printf("CANNOT CONVERT %s!\n",filename);
		}
		return;
	}
ok:

	// 5 to 6
	if (mapversion == NEWMAPVERSION-2)
	{
		for(i=0;i<numsectors;i++)
		{
			sector[i].wallptr = oldsector[i].wallptr;
			sector[i].wallnum = oldsector[i].wallnum;
			sector[i].ceilingpicnum = oldsector[i].ceilingpicnum;
			sector[i].floorpicnum = oldsector[i].floorpicnum;
			sector[i].ceilingheinum = oldsector[i].ceilingheinum;
			sector[i].floorheinum = oldsector[i].floorheinum;
			sector[i].ceilingz = oldsector[i].ceilingz;
			sector[i].floorz = oldsector[i].floorz;
			sector[i].ceilingshade = oldsector[i].ceilingshade;
			sector[i].floorshade = oldsector[i].floorshade;
			sector[i].ceilingxpanning = oldsector[i].ceilingxpanning;
			sector[i].floorxpanning = oldsector[i].floorxpanning;
			sector[i].ceilingypanning = oldsector[i].ceilingypanning;
			sector[i].floorypanning = oldsector[i].floorypanning;
			sector[i].ceilingstat = oldsector[i].ceilingstat;
			sector[i].floorstat = oldsector[i].floorstat;
			sector[i].ceilingpal = oldsector[i].ceilingpal;
			sector[i].floorpal = oldsector[i].floorpal;
			sector[i].visibility = oldsector[i].visibility;
			sector[i].lotag = oldsector[i].lotag;
			sector[i].hitag = oldsector[i].hitag;
			sector[i].extra = oldsector[i].extra;
		}
		for(i=0;i<numwalls;i++)
		{
			wall[i].x = oldwall[i].x;
			wall[i].y = oldwall[i].y;
			wall[i].point2 = oldwall[i].point2;
			wall[i].nextsector = oldwall[i].nextsector1;
			wall[i].nextwall = oldwall[i].nextwall1;
			wall[i].picnum = oldwall[i].picnum;
			wall[i].overpicnum = oldwall[i].overpicnum;
			wall[i].shade = oldwall[i].shade;
			wall[i].pal = sector[sectorofwall((short)i)].floorpal;
			wall[i].cstat = oldwall[i].cstat;
			wall[i].xrepeat = oldwall[i].xrepeat;
			wall[i].yrepeat = oldwall[i].yrepeat;
			wall[i].xpanning = oldwall[i].xpanning;
			wall[i].ypanning = oldwall[i].ypanning;
			wall[i].lotag = oldwall[i].lotag;
			wall[i].hitag = oldwall[i].hitag;
			wall[i].extra = oldwall[i].extra;
		}
		for(i=0;i<numsprites;i++)
		{
			sprite[i].x = oldsprite[i].x;
			sprite[i].y = oldsprite[i].y;
			sprite[i].z = oldsprite[i].z;
			sprite[i].cstat = oldsprite[i].cstat;
			sprite[i].shade = oldsprite[i].shade;

			j = oldsprite[i].sectnum;
			if ((oldsector[j].ceilingstat&1) > 0)
				sprite[i].pal = oldsector[j].ceilingpal;
			else
				sprite[i].pal = oldsector[j].floorpal;

			sprite[i].clipdist = 32;
			sprite[i].xrepeat = oldsprite[i].xrepeat;
			sprite[i].yrepeat = oldsprite[i].yrepeat;
			sprite[i].xoffset = 0;
			sprite[i].yoffset = 0;
			sprite[i].picnum = oldsprite[i].picnum;
			sprite[i].ang = oldsprite[i].ang;
			sprite[i].xvel = oldsprite[i].xvel;
			sprite[i].yvel = oldsprite[i].yvel;
			sprite[i].zvel = oldsprite[i].zvel;
			sprite[i].owner = oldsprite[i].owner;
			sprite[i].sectnum = oldsprite[i].sectnum;
			sprite[i].statnum = oldsprite[i].statnum;
			sprite[i].lotag = oldsprite[i].lotag;
			sprite[i].hitag = oldsprite[i].hitag;
			sprite[i].extra = oldsprite[i].extra;
		}

		mapversion = NEWMAPVERSION-1;
	}

	// 6 to 7

	for(i=0;i<numsectors;i++)
	{
		newsector[i].ceilingz = sector[i].ceilingz;
		newsector[i].floorz = sector[i].floorz;
		newsector[i].wallptr = sector[i].wallptr;
		newsector[i].wallnum = sector[i].wallnum;
		newsector[i].ceilingpicnum = convert_pics(sector[i].ceilingpicnum);
		newsector[i].ceilingheinum = max(min(((long)sector[i].ceilingheinum)<<5,32767),-32768);
		if ((sector[i].ceilingstat&2) == 0) newsector[i].ceilingheinum = 0;
		newsector[i].ceilingshade = sector[i].ceilingshade;
		newsector[i].ceilingpal = sector[i].ceilingpal;
		newsector[i].ceilingxpanning = sector[i].ceilingxpanning;
		newsector[i].ceilingypanning = sector[i].ceilingypanning;
		newsector[i].floorpicnum = convert_pics(sector[i].floorpicnum);
		newsector[i].floorheinum = max(min(((long)sector[i].floorheinum)<<5,32767),-32768);
		if ((sector[i].floorstat&2) == 0) newsector[i].floorheinum = 0;
		newsector[i].floorshade = sector[i].floorshade;
		newsector[i].floorpal = sector[i].floorpal;
		newsector[i].floorxpanning = sector[i].floorxpanning;
		newsector[i].floorypanning = sector[i].floorypanning;
		newsector[i].ceilingstat = sector[i].ceilingstat;
		newsector[i].floorstat = sector[i].floorstat;
		newsector[i].visibility = sector[i].visibility;
		newsector[i].filler = 0;
		newsector[i].lotag = sector[i].lotag;
		newsector[i].hitag = sector[i].hitag;
		newsector[i].extra = sector[i].extra;
	}
	for(i=0;i<numwalls;i++)
	{
		newwall[i].x = wall[i].x;
		newwall[i].y = wall[i].y;
		newwall[i].point2 = wall[i].point2;
		newwall[i].nextwall = wall[i].nextwall;
		newwall[i].nextsector = wall[i].nextsector;
		newwall[i].cstat = wall[i].cstat;
		newwall[i].picnum = convert_pics(wall[i].picnum);
		newwall[i].overpicnum = convert_pics(wall[i].overpicnum);
		newwall[i].shade = wall[i].shade;
		newwall[i].pal = wall[i].pal;
		newwall[i].xrepeat = wall[i].xrepeat;
		newwall[i].yrepeat = wall[i].yrepeat;
		newwall[i].xpanning = wall[i].xpanning;
		newwall[i].ypanning = wall[i].ypanning;
		newwall[i].lotag = wall[i].lotag;
		newwall[i].hitag = wall[i].hitag;
		newwall[i].extra = wall[i].extra;
	}
	for(i=0;i<numsprites;i++)
	{
		newsprite[i].x = sprite[i].x;
		newsprite[i].y = sprite[i].y;
		newsprite[i].z = sprite[i].z;
		newsprite[i].cstat = sprite[i].cstat;
		newsprite[i].picnum = convert_pics(sprite[i].picnum);
		newsprite[i].shade = sprite[i].shade;
		newsprite[i].pal = sprite[i].pal;
		newsprite[i].clipdist = sprite[i].clipdist;
		newsprite[i].filler = 0;
		newsprite[i].xrepeat = sprite[i].xrepeat;
		newsprite[i].yrepeat = sprite[i].yrepeat;
		newsprite[i].xoffset = sprite[i].xoffset;
		newsprite[i].yoffset = sprite[i].yoffset;
		newsprite[i].sectnum = sprite[i].sectnum;
		newsprite[i].statnum = sprite[i].statnum;
		newsprite[i].ang = sprite[i].ang;
		newsprite[i].owner = sprite[i].owner;
		newsprite[i].xvel = sprite[i].xvel;
		newsprite[i].yvel = sprite[i].yvel;
		newsprite[i].zvel = sprite[i].zvel;
		newsprite[i].lotag = sprite[i].lotag;
		newsprite[i].hitag = sprite[i].hitag;
		newsprite[i].extra = sprite[i].extra;
	}

	mapversion = NEWMAPVERSION;
	if (savenewboard(filename) == 0)
		printf("%s successfully converted.\n",filename);
	else
		printf("%s NOT converted for some stupid reason.\n",filename);
}

loadoldboard(char *filename)
{
	long i, fil;

	if ((fil = open(filename,O_BINARY|O_RDWR,S_IREAD)) == -1)
		return(-1);

	read(fil,&mapversion,4);
	if (mapversion != NEWMAPVERSION-1)
	{
		close(fil);
		return(-2);
	}
	read(fil,&posx,4);
	read(fil,&posy,4);
	read(fil,&posz,4);
	read(fil,&ang,2);
	read(fil,&cursectnum,2);

	read(fil,&numsectors,2);
	read(fil,&sector[0],sizeof(sectortype)*numsectors);

	read(fil,&numwalls,2);
	read(fil,&wall[0],sizeof(walltype)*numwalls);

	read(fil,&numsprites,2);
	read(fil,&sprite[0],sizeof(spritetype)*numsprites);

	close(fil);
	return(0);
}

loadreallyoldboard(char *filename)
{
	long i, fil;

	if ((fil = open(filename,O_BINARY|O_RDWR,S_IREAD)) == -1)
		return(-1);

	read(fil,&mapversion,4);
	if (mapversion != NEWMAPVERSION-2)
	{
		close(fil);
		return(-2);
	}
	read(fil,&posx,4);
	read(fil,&posy,4);
	read(fil,&posz,4);
	read(fil,&ang,2);
	read(fil,&cursectnum,2);

	read(fil,&numsectors,2);
	read(fil,&oldsector[0],sizeof(oldsectortype)*numsectors);

	read(fil,&numwalls,2);
	read(fil,&oldwall[0],sizeof(oldwalltype)*numwalls);

	read(fil,&numsprites,2);
	read(fil,&oldsprite[0],sizeof(oldspritetype)*numsprites);

	close(fil);
	return(0);
}

savenewboard(char *filename)
{
	long i, fil;

	if ((fil = open(filename,O_BINARY|O_TRUNC|O_CREAT|O_WRONLY,S_IWRITE)) == -1)
		return(-1);

		 //Version control
	write(fil,&mapversion,4);

		 //Starting position
	write(fil,&posx,4);
	write(fil,&posy,4);
	write(fil,&posz,4);
	write(fil,&ang,2);
	write(fil,&cursectnum,2);

		//Sectors
	write(fil,&numsectors,2);
	write(fil,&newsector[0],sizeof(newsectortype)*numsectors);

		//Walls
	write(fil,&numwalls,2);
	write(fil,&newwall[0],sizeof(newwalltype)*numwalls);

		//Sprites
	write(fil,&numsprites,2);
	write(fil,&newsprite[0],sizeof(newspritetype)*numsprites);

	close(fil);

	return(0);
}

struct find_t
{
	long handle;
    struct _finddata_t data;
	char name[512];
};
int _dos_findfirst(char *filename, int x, struct find_t *f);
int _dos_findnext(struct find_t *f);


int _dos_findfirst(char *filename, int x, struct find_t *f)
{
    long rc = _findfirst(filename, &f->data);
    f->handle = rc;
    if (rc != -1)
    {
        strncpy(f->name, f->data.name, sizeof (f->name) - 1);
        f->name[sizeof (f->name) - 1] = '\0';
        return(0);
    }
    return(1);
}

int _dos_findnext(struct find_t *f)
{
    int rc = 0;
    if (f->handle == -1)
        return(1);   /* invalid handle. */

    rc = _findnext(f->handle, &f->data);
    if (rc == -1)
    {
        _findclose(f->handle);
        f->handle = -1;
        return(1);
    }

    strncpy(f->name, f->data.name, sizeof (f->name) - 1);
    f->name[sizeof (f->name) - 1] = '\0';
    return(0);
}

#include <Windows.h>
int getfilenames(char *kind)
{
	short type;
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind;

	if (strcmp(kind,"SUBD") == 0)
	{
		if ((hFind = FindFirstFile("*.*", &FindFileData)) == INVALID_HANDLE_VALUE)
			return(-1);
		type = 1;
	}
	else
	{
		if ((hFind = FindFirstFile(kind, &FindFileData)) == INVALID_HANDLE_VALUE)
			return(-1);
		type = 0;
	}
	do
	{
		int file_is_dir = (FindFileData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY);
		if ((!type && !file_is_dir) || (type && file_is_dir))
		{
			if ((FindFileData.cFileName[0] != '.') || (FindFileData.cFileName[1] != 0))
			{
				int len = strlen(FindFileData.cFileName);
				if (len <= 12) strcpy(menuname[menunamecnt],FindFileData.cFileName);
				else strcpy(menuname[menunamecnt],FindFileData.cAlternateFileName);
				menuname[menunamecnt][MENUNAME_SIZE] = type;
				menunamecnt++;
			}
		}
	}
	while (FindNextFile(hFind,&FindFileData));

	FindClose(hFind);

	return(0);
}

sortfilenames()
{
	char sortbuffer[40], menuname_i__k_, menuname_j__k_;
	long i, j, k, m;

	for(i=1;i<menunamecnt;i++)
		for(j=0;j<i;j++)
		{
			k = 0;
			menuname_i__k_ = toupper(menuname[i][k]);
			menuname_j__k_ = toupper(menuname[j][k]);
			while ((menuname_i__k_ == menuname_j__k_) && (menuname_i__k_ != 0) && (menuname_j__k_ != 0))
			{
				k++;
				menuname_i__k_ = toupper(menuname[i][k]);
				menuname_j__k_ = toupper(menuname[j][k]);
			}
			if (menuname_i__k_ < menuname_j__k_)
			{
				memcpy(&sortbuffer[0],&menuname[i][0],sizeof(menuname[0]));
				memcpy(&menuname[i][0],&menuname[j][0],sizeof(menuname[0]));
				memcpy(&menuname[j][0],&sortbuffer[0],sizeof(menuname[0]));
			}
		}
}
